@@ -47,8 +47,8 @@
 #include <stdlib.h>
 #include <zlib.h>
 
-bool Uic::hasKDEwidget = FALSE;
-bool Uic::isMainWindow = FALSE;
+bool Uic::hasKDEwidget = false;
+bool Uic::isMainWindow = false;
 RubyIndent Uic::indent;
 
 
@@ -128,7 +128,7 @@ bool Uic::isEmptyFunction( const QString& fname )
 	return body.simplifyWhiteSpace().isEmpty();
     }
 	// For now ruby functions are always empty, until a rubyeditor Qt Designer plugin exists..
-    return TRUE;
+    return true;
 }
 
 
@@ -147,10 +147,10 @@ Uic::Uic( const QString &fn, QTextStream &outStream, QDomDocument doc,
 {
 	Uic::hasKDEwidget = useKDE;
     fileName = fn;
-    writeSlotImpl = TRUE;
+    writeSlotImpl = true;
     defMargin = BOXLAYOUT_DEFAULT_MARGIN;
     defSpacing = BOXLAYOUT_DEFAULT_SPACING;
-    externPixmaps = FALSE;
+    externPixmaps = false;
 
     item_used = cg_used = pal_used = 0;
 
@@ -161,6 +161,7 @@ Uic::Uic( const QString &fn, QTextStream &outStream, QDomDocument doc,
     pixmapLoaderFunction = getPixmapLoaderFunction( doc.firstChild().toElement() );
     nameOfClass = getFormClassName( doc.firstChild().toElement() );
 
+    uiFileVersion = doc.firstChild().toElement().attribute("version");
     stdsetdef = toBool( doc.firstChild().toElement().attribute("stdsetdef") );
 
     QDomElement e = doc.firstChild().firstChild().toElement();
@@ -169,7 +170,7 @@ Uic::Uic( const QString &fn, QTextStream &outStream, QDomDocument doc,
 	if ( e.tagName() == "widget" ) {
 	    widget = e;
 	} else if ( e.tagName() == "pixmapinproject" ) {
-	    externPixmaps = TRUE;
+	    externPixmaps = true;
 	} else if ( e.tagName() == "layoutdefaults" ) {
 	    defSpacing = e.attribute( "spacing", QString::number( defSpacing ) ).toInt();
 	    defMargin = e.attribute( "margin", QString::number( defMargin ) ).toInt();
@@ -230,17 +231,19 @@ QString Uic::getClassName( const QDomElement& e )
 	s = "Qt::ToolBar";
     else if ( s.isEmpty() && e.tagName() == "menubar" )
 	s = "Qt::MenuBar";
-//    else if( WidgetDatabase::idFromClassName( s ) == -1)
-//	  return s;
     else
     {
 	QRegExp r("^([QK])(\\S+)");
         if( r.search( s ) != -1 ) {
 	    	if (r.cap(1) == "K") {
-				hasKDEwidget = TRUE;
+				hasKDEwidget = true;
 	    		s = "KDE::" + r.cap(2);
 			} else {
-	    		s = "Qt::" + r.cap(2);
+				if (s.startsWith("Qext")) {
+	    			s = "Qext::" + r.cap(2).mid(4);
+				} else {
+	    			s = "Qt::" + r.cap(2);
+				}
 			}
 		}
     }
@@ -256,9 +259,9 @@ bool Uic::isFrameworkCodeGenerated( const QDomElement& e )
 {
     QDomElement n = getObjectProperty( e, "frameworkCode" );
     if ( n.attribute("name") == "frameworkCode" &&
-	 !DomTool::elementToVariant( n.firstChild().toElement(), QVariant( TRUE, 0 ) ).toBool() )
-	return FALSE;
-    return TRUE;
+	 !DomTool::elementToVariant( n.firstChild().toElement(), QVariant( true, 0 ) ).toBool() )
+	return false;
+    return true;
 }
 
 /*! Extracts an object name from \a e. It's stored in the 'name'
@@ -359,12 +362,14 @@ void Uic::createActionImpl( const QDomElement &n, const QString &parent )
     for ( QDomElement ae = n; !ae.isNull(); ae = ae.nextSibling().toElement() ) {
 	QString objName = registerObject( getObjectName( ae ) );
 	if ( ae.tagName() == "action" )
-	    out << indent << objName << "= Qt::Action.new(" << parent << ", \"" << objName << "\")" << endl;
+	    out << indent << objName << "= Qt::Action.new(" << parent << ", \"" << objName.mid(1) << "\")" << endl;
 	else if ( ae.tagName() == "actiongroup" )
-	    out << indent << objName << "= Qt::ActionGroup.new(" << parent << ", \"" << objName << "\")" << endl;
+	    out << indent << objName << "= Qt::ActionGroup.new(" << parent << ", \"" << objName.mid(1) << "\")" << endl;
 	else
 	    continue;
-	bool subActionsDone = FALSE;
+	bool subActionsDone = false;
+	bool hasMenuText = false;
+	QString actionText;
 	for ( QDomElement n2 = ae.firstChild().toElement(); !n2.isNull(); n2 = n2.nextSibling().toElement() ) {
 	    if ( n2.tagName() == "property" ) {
 		bool stdset = stdsetdef;
@@ -373,7 +378,7 @@ void Uic::createActionImpl( const QDomElement &n, const QString &parent )
 		QString prop = n2.attribute("name");
 		if ( prop == "name" )
 		    continue;
-		QString value = setObjectProperty( "Qt::Action.new", objName, prop, n2.firstChild().toElement(), stdset );
+		QString value = setObjectProperty( "Qt::Action", objName, prop, n2.firstChild().toElement(), stdset );
 		if ( value.isEmpty() )
 		    continue;
 			
@@ -385,6 +390,11 @@ void Uic::createActionImpl( const QDomElement &n, const QString &parent )
 			call += "Qt::Variant.new(" + value + "))";
 		}
 		
+		if (prop == "menuText")
+		    hasMenuText = true;
+		else if (prop == "text")
+		    actionText = value;
+		
 		if ( n2.firstChild().toElement().tagName() == "string" ) {
 		    trout << indent << call << endl;
 		} else {
@@ -392,9 +402,12 @@ void Uic::createActionImpl( const QDomElement &n, const QString &parent )
 		}
 	    } else if ( !subActionsDone && ( n2.tagName() == "actiongroup" || n2.tagName() == "action" ) ) {
 		createActionImpl( n2, objName );
-		subActionsDone = TRUE;
+		subActionsDone = true;
 	    }
 	}
+	// workaround for loading pre-3.3 files expecting bogus QAction behavior
+	if (!hasMenuText && !actionText.isEmpty() && uiFileVersion < "3.3")
+	    trout << indent << objName << ".setMenuText(" << actionText << ")" << endl;
     }
 }
 
@@ -435,7 +448,7 @@ void Uic::createToolbarImpl( const QDomElement &n, const QString &parentClass, c
 		if ( n2.attribute( "class" ) != "Spacer" ) {
 			createObjectImpl( n2, "Qt::ToolBar", objName );
 		} else {
-		    QString child = createSpacerImpl( n, parentClass, parent, objName );
+		    QString child = createSpacerImpl( n2, parentClass, parent, objName );
 		    out << indent << "Qt::Application.sendPostedEvents( " << objName
 			<< ", Qt::Event::ChildInserted)" << endl;
 		    out << indent <<  objName << ".boxLayout().addItem(" << child << ")" << endl;
@@ -448,7 +461,7 @@ void Uic::createToolbarImpl( const QDomElement &n, const QString &parentClass, c
 void Uic::createMenuBarImpl( const QDomElement &n, const QString &parentClass, const QString &parent )
 {
     QString objName = "@" + getObjectName( n );
-    out << indent << objName << " = Qt::MenuBar.new( self, \"" << objName << "\" )" << endl;
+    out << indent << objName << " = Qt::MenuBar.new( self, \"" << objName.mid(1) << "\" )" << endl;
     createObjectImpl( n, parentClass, parent );
     int i = 0;
     QDomElement c = n.firstChild().toElement();
@@ -459,7 +472,11 @@ void Uic::createMenuBarImpl( const QDomElement &n, const QString &parentClass, c
 	    out << indent << itemName << " = Qt::PopupMenu.new( self )" << endl;
 	    createPopupMenuImpl( c, parentClass, itemName );
 	    out << indent << objName << ".insertItem( \"\", " << itemName << ", " << i << " )" << endl;
-	    trout << indent << objName << ".findItem( " << i << " ).setText( " << trcall( c.attribute( "text" ) ) << " )" << endl;
+		QString findItem(objName + ".findItem(%1)");
+	    findItem = findItem.arg(i);
+	    trout << indent << "if !" << findItem << ".nil?" << endl;
+	    trout << indent << indent << findItem << ".setText( " << trcall( c.attribute( "text" ) ) << " )" << endl;
+	    trout << indent << "end" << endl;
 	} else if ( c.tagName() == "separator" ) {
 	    out << endl;
 	    out << indent << objName << ".insertSeparator( " << i << " )" << endl;
@@ -471,16 +488,18 @@ void Uic::createMenuBarImpl( const QDomElement &n, const QString &parentClass, c
 
 void Uic::createPopupMenuImpl( const QDomElement &e, const QString &parentClass, const QString &parent )
 {
+	int i = 0;
     for ( QDomElement n = e.firstChild().toElement(); !n.isNull(); n = n.nextSibling().toElement() ) {
-	if ( n.tagName() == "action" ) {
+	if ( n.tagName() == "action" || n.tagName() == "actiongroup" ) {
 	    QDomElement n2 = n.nextSibling().toElement();
 	    if ( n2.tagName() == "item" ) { // the action has a sub menu
 		QString itemName = "@" + n2.attribute( "name" );
 		QString itemText = n2.attribute( "text" );
 		out << indent << itemName << " = Qt::PopupMenu.new( self )" << endl;
-		out << indent << parent << ".setAccel( tr( \"" << n2.attribute( "accel" ) << "\" ), " << endl;
 		out << indent << indent << parent << ".insertItem( @" << n.attribute( "name" ) << ".iconSet(),";
-		out << trcall( itemText ) << ", " << itemName << " ) )" << endl;
+		out << trcall( itemText ) << ", " << itemName << " )" << endl;
+		trout << indent << parent << ".changeItem( " << parent << ".idAt( " << i << " ), ";
+		trout << trcall( itemText ) << " )" << endl;
 		createPopupMenuImpl( n2, parentClass, itemName );
 		n = n2;
 	    } else {
@@ -489,6 +508,7 @@ void Uic::createPopupMenuImpl( const QDomElement &e, const QString &parentClass,
 	} else if ( n.tagName() == "separator" ) {
 	    out << indent << parent << ".insertSeparator()" << endl;
 	}
+	++i;
     }
 }
 
@@ -592,7 +612,7 @@ QString Uic::createListViewItemImpl( const QDomElement &e, const QString &parent
 			// This is here to make the ruby generated names for 'item', 
 			// the same as the original C++
 			item = registerObject( "item" );
-			item_used = TRUE;
+			item_used = true;
 		}
 	item = "item";
 	s = indent + item + " = ";
@@ -651,7 +671,7 @@ QString Uic::createListViewColumnImpl( const QDomElement &e, const QString &pare
     QString txt;
     QString com;
     QString pix;
-    bool clickable = FALSE, resizeable = FALSE;
+    bool clickable = false, resizeable = false;
     while ( !n.isNull() ) {
 	if ( n.tagName() == "property" ) {
 	    QString attrib = n.attribute("name");
@@ -799,7 +819,7 @@ QString Uic::createLayoutImpl( const QDomElement &e, const QString& parentClass,
 	    if ( !DomTool::hasProperty( e, "margin" ) )
 		margin = "0";
 	}
-	out << ", " << optcells << margin << ", " << spacing << ", '" << objName << "')" << endl;
+	out << ", " << optcells << margin << ", " << spacing << ", '" << objName.mid(1) << "')" << endl;
     }
     if ( !resizeMode.isEmpty() )
 	out << indent << objName << ".setResizeMode( Qt::Layout::" << resizeMode << " )" << endl;
@@ -862,7 +882,7 @@ QString Uic::createSpacerImpl( const QDomElement &e, const QString& /*parentClas
     QDomElement n;
     QString objClass, objName;
     objClass = e.tagName();
-    objName = registerObject( "spacer" );
+    objName = registerObject( getObjectName( e ) );
 
     QSize size = DomTool::readProperty( e, "sizeHint", QSize( 0, 0 ) ).toSize();
     QString sizeType = DomTool::readProperty( e, "sizeType", "Expanding" ).toString();
@@ -958,8 +978,8 @@ bool Uic::isWidgetInTable( const QDomElement& e, const QString& connection, cons
     QString conn = getDatabaseInfo( e, "connection" );
     QString tab = getDatabaseInfo( e, "table" );
     if ( conn == connection && tab == table )
-	return TRUE;
-    return FALSE;
+	return true;
+    return false;
 }
 
 /*!
@@ -1073,9 +1093,9 @@ QString Uic::createObjectInstance( const QString& objClass, const QString& paren
 {
 
     if ( objClass.mid( 4 ) == "ComboBox" ) {
-	return objClass + ".new(false, " + parent + ", \"" + objName + "\")";
+	return objClass + ".new(false, " + parent + ", \"" + objName.mid(1) + "\")";
     }
-    return objClass + ".new(" + parent + ", \"" + objName + "\")";
+    return objClass + ".new(" + parent + ", \"" + objName.mid(1) + "\")";
 }
 
 bool Uic::isLayout( const QString& name ) const
